@@ -7,6 +7,7 @@ import (
 	"io/ioutil"
 	"os"
 	"os/exec"
+	"reflect"
 	"testing"
 )
 
@@ -196,3 +197,46 @@ func TestScanMemCgoPointer(t *testing.T) {
 		t.Errorf("ScanMem panicked: %s", err)
 	}
 }
+
+func TestRule(t *testing.T) {
+	r := makeRules(t, `
+		rule t1 : tag1 { meta: author = "Author One" strings: $a = "abc" fullword condition: $a }
+        rule t2 : tag2 x y { meta: author = "Author Two" strings: $b = "def" condition: $b }
+        rule t3 : tag3 x y z { meta: author = "Author Three" strings: $c = "ghi" condition: $c }
+		rule t4 { strings: $d = "qwe" condition: $d }`)
+	for _, r := range r.GetRules() {
+		t.Logf("%s:%s %#v", r.Namespace(), r.Identifier(), r.Tags())
+		switch r.Identifier() {
+		case "t1":
+			if !reflect.DeepEqual(r.Tags(), []string{"tag1"}) {
+				t.Error("Got wrong tags for t1")
+			}
+			if !reflect.DeepEqual(r.Metas(), map[string]interface{}{"author": "Author One"}) {
+				t.Error("Got wrong meta variables for t1")
+			}
+		case "t2":
+			if !reflect.DeepEqual(r.Tags(), []string{"tag2", "x", "y"}) {
+				t.Error("Got wrong tags for t2")
+			}
+			if !reflect.DeepEqual(r.Metas(), map[string]interface{}{"author": "Author Two"}) {
+				t.Error("Got wrong meta variables for t2")
+			}
+		case "t3":
+			if !reflect.DeepEqual(r.Tags(), []string{"tag3", "x", "y", "z"}) {
+				t.Error("Got wrong tags for t3")
+			}
+			if !reflect.DeepEqual(r.Metas(), map[string]interface{}{"author": "Author Three"}) {
+				t.Error("Got wrong meta variables for t3")
+			}
+		case "t4":
+			if len(r.Tags()) != 0 {
+				t.Error("Got tags for t4")
+			}
+			if !reflect.DeepEqual(r.Metas(), map[string]interface{}{}) {
+				t.Error("Got wrong meta variables for t4")
+			}
+		default:
+			t.Errorf("Found unexpected rule name: %#v", r.Identifier())
+		}
+	}
+}
