@@ -14,13 +14,21 @@ package yara
 #include <yara.h>
 
 #ifdef _WIN32
+// Helper function that is merely used to cast fd from int to HANDLE.
+// CGO treats HANDLE (void*) to an unsafe.Pointer. This confuses the
+// go1.4 garbage collector, leading to runtime errors such as:
+//
+// runtime: garbage collector found invalid heap pointer *(0x5b80ff14+0x4)=0xa0 s=nil
 int _yr_rules_scan_fd(
     YR_RULES* rules,
     int fd,
     int flags,
     YR_CALLBACK_FUNC callback,
     void* user_data,
-    int timeout);
+    int timeout)
+{
+  return yr_rules_scan_fd(rules, (YR_FILE_DESCRIPTOR)fd, flags, callback, user_data, timeout);
+}
 #else
 #define _yr_rules_scan_fd yr_rules_scan_fd
 #endif
@@ -28,7 +36,7 @@ int _yr_rules_scan_fd(
 size_t streamRead(void* ptr, size_t size, size_t nmemb, void* user_data);
 size_t streamWrite(void* ptr, size_t size, size_t nmemb, void* user_data);
 
-int rules_callback(int message, void *message_data, void *user_data);
+int stdScanCallback(int, void*, void*);
 */
 import "C"
 import (
@@ -46,9 +54,10 @@ func (r *Rules) ScanFileDescriptor(fd uintptr, flags ScanFlags, timeout time.Dur
 		r.cptr,
 		C.int(fd),
 		C.int(flags),
-		C.YR_CALLBACK_FUNC(C.rules_callback),
-		unsafe.Pointer(id),
+		C.YR_CALLBACK_FUNC(C.stdScanCallback),
+		unsafe.Pointer(&id),
 		C.int(timeout/time.Second)))
+	keepAlive(id)
 	keepAlive(r)
 	return
 }
@@ -58,12 +67,15 @@ func (r *Rules) Write(wr io.Writer) (err error) {
 	id := callbackData.Put(wr)
 	defer callbackData.Delete(id)
 
-	stream := (*C.YR_STREAM)(C.malloc((C.sizeof_YR_STREAM)))
-	defer C.free(unsafe.Pointer(stream))
-	stream.user_data = unsafe.Pointer(id)
-	stream.write = C.YR_STREAM_WRITE_FUNC(C.streamWrite)
-
-	err = newError(C.yr_rules_save_stream(r.cptr, stream))
+	stream := C.YR_STREAM{
+		write: C.YR_STREAM_WRITE_FUNC(C.streamWrite),
+		// The complaint from go vet about possible misuse of
+		// unsafe.Pointer is wrong: user_data will be interpreted as
+		// an uintptr on the other side of the callback
+		user_data: unsafe.Pointer(id),
+	}
+	err = newError(C.yr_rules_save_stream(r.cptr, &stream))
+	keepAlive(id)
 	keepAlive(r)
 	return
 }
@@ -74,15 +86,17 @@ func ReadRules(rd io.Reader) (*Rules, error) {
 	id := callbackData.Put(rd)
 	defer callbackData.Delete(id)
 
-	stream := (*C.YR_STREAM)(C.malloc((C.sizeof_YR_STREAM)))
-	defer C.free(unsafe.Pointer(stream))
-	stream.user_data = unsafe.Pointer(id)
-	stream.read = C.YR_STREAM_READ_FUNC(C.streamRead)
-
-	if err := newError(C.yr_rules_load_stream(stream,
+	stream := C.YR_STREAM{
+		read: C.YR_STREAM_READ_FUNC(C.streamRead),
+		// The complaint from go vet about possible misuse of
+		// unsafe.Pointer is wrong, see above.
+		user_data: unsafe.Pointer(id),
+	}
+	if err := newError(C.yr_rules_load_stream(&stream,
 		&(r.rules.cptr))); err != nil {
 		return nil, err
 	}
 	runtime.SetFinalizer(r.rules, (*rules).finalize)
+	keepAlive(id)
 	return r, nil
 }
